==> grabberlib2-0.1.17-py3-none-any.whl: grabber/core/sources/hotgirl_china.py <==
import pathlib
from urllib.parse import urljoin

from boltons.setutils import IndexedSet
from bs4 import Tag
from telegraph import Telegraph
from tqdm import tqdm

from database.models import ExtractedPage
from database.repositories.extracted_page import ExtractedPageRepository
from grabber.core.utils import (
    build_unique_img_urls,
    get_all_posts_for_url,
    get_soup,
    query_mapping,
    headers_mapping,
    get_tags,
    run_downloader,
    send_post_to_telegram,
)


async def get_images_from_pagination(url: str, headers: dict[str, str] | None = None) -> list[str]:
    pagination_query = "nav.pagination.group div.wp-pagenavi"
    first_page_query = f"{pagination_query} span.current"
    last_page_query = f"{pagination_query} a.last"
    pages_count_query = f"{pagination_query} a"

    source_urls = IndexedSet()
    source_urls.add(url)
    soup = await get_soup(url, headers=headers)
    pagination_set = soup.select(pages_count_query)

    if not pagination_set:
        return list(source_urls)

    first_page = soup.select(first_page_query)[0]
    first_page_number = int(first_page.text)

    last_page = soup.select(last_page_query)
    if last_page:
        last_page = last_page[0]
        last_page = last_page.attrs["href"].split("/", maxsplit=4)[-1].split("/")[0]
        last_page_number = int(last_page)
    else:
        pages = []

        for a in soup.select(pages_count_query):
            text: str = a.get_text()
            if text.isdigit():
                pages.append(int(text))
        last_page_number = max(pages)

    for index in range(first_page_number, last_page_number + 1):
        if index == 1:
            continue

        target_url = urljoin(url, f"{index}/")
        source_urls.add(target_url)

    return list(source_urls)


async def get_sources_for_hotgirl_china(
    sources: list[str],
    entity: str,
    telegraph_client: Telegraph | None = None,
    final_dest: str | pathlib.Path = "",
    save_to_telegraph: bool | None = False,
    is_tag: bool | None = False,
    limit: int | None = None,
    **kwargs,
) -> None:
    tqdm_sources_iterable = tqdm(
        sources,
        total=len(sources),
        desc="Retrieving URLs...",
    )
    query, src_attr = query_mapping[entity]
    headers = headers_mapping.get(entity, None)
    page_title = ""
    title_folder_mapping = {}
    posts_sent_counter = 0
    titles = IndexedSet()
    repository = ExtractedPageRepository(model=ExtractedPage)
    ordered_unique_img_urls = None
    all_sources = []

    if final_dest:
        final_dest = pathlib.Path(f"{final_dest} - {entity}")
        if not final_dest.exists():
            final_dest.mkdir(parents=True, exist_ok=True)

    for source_url in tqdm_sources_iterable:
        all_sources += await get_all_posts_for_url(repository=repository, url=source_url)

        if source_url in all_sources:
            tqdm_sources_iterable.set_description(
                f"Skipping {source_url} since it was already posted"
            )
            continue

        folder_name = ""
        urls = [*await get_images_from_pagination(url=source_url, headers=headers)]
        image_tags: list[Tag] = []

        for index, url in enumerate(urls):
            tags, soup = await get_tags(
                url,
                headers=headers,
                query=query,
            )
            image_tags.extend(tags or [])

            if index == 0:
                folder_name = (
                    soup.find("title")
                    .get_text(strip=True)
                    .split("| Hot Girl China")[0]
                    .split("/mitaku.net/")[0]
                    .strip()
                    .rstrip()
                )
                page_title = folder_name
                titles.add(page_title)

            if not page_title:
                page_title = (
                    soup.find("title")
                    .get_text(strip=True)
                    .split("| Hot Girl China")[0]
                    .split("/mitaku.net/")[0]
                    .strip()
                    .rstrip()
                )
                titles.add(page_title)

        ordered_unique_img_urls = await build_unique_img_urls(image_tags, src_attr)
        tqdm_sources_iterable.set_description(f"Finished retrieving images for {page_title}")

        if final_dest:
            final_dest = pathlib.Path(f"{final_dest} - {entity}")
            if not final_dest.exists():
                final_dest.mkdir(parents=True, exist_ok=True)

            folder_name = page_title
            title_dest = final_dest / folder_name
            if not title_dest.exists():
                title_dest.mkdir(parents=True, exist_ok=True)
            title_folder_mapping[page_title] = (ordered_unique_img_urls, title_dest)

        if save_to_telegraph:
            all_sources = await send_post_to_telegram(
                ordered_unique_img_urls=ordered_unique_img_urls,
                page_title=page_title,
                telegraph_client=telegraph_client,
                posts_sent_counter=posts_sent_counter,
                tqdm_sources_iterable=tqdm_sources_iterable,
                all_sources=all_sources,
                source_url=source_url,
                entity=entity,
            )
        page_title = ""

    if final_dest and ordered_unique_img_urls:
        await run_downloader(
            final_dest=final_dest,
            page_title=page_title,
            unique_img_urls=ordered_unique_img_urls,
            titles=titles,
            title_folder_mapping=title_folder_mapping,
            headers=headers,
        )

==> grabberlib2-0.1.20-py3-none-any.whl: grabber/core/sources/hotgirl_china.py <==
import pathlib
from urllib.parse import urljoin

from boltons.setutils import IndexedSet
from bs4 import Tag
from telegraph import Telegraph
from tqdm import tqdm

from database.models import ExtractedPage
from database.repositories.extracted_page import ExtractedPageRepository
from grabber.core.utils import (
    build_unique_img_urls,
    get_all_posts_for_url,
    get_soup,
    query_mapping,
    headers_mapping,
    get_tags,
    run_downloader,
    send_post_to_telegram,
)


async def get_images_from_pagination(url: str, headers: dict[str, str] | None = None) -> list[str]:
    pagination_query = "nav.pagination.group div.wp-pagenavi"
    first_page_query = f"{pagination_query} span.current"
    last_page_query = f"{pagination_query} a.last"
    pages_count_query = f"{pagination_query} a"

    source_urls = IndexedSet()
    source_urls.add(url)
    soup = await get_soup(url, headers=headers)
    pagination_set = soup.select(pages_count_query)

    if not pagination_set:
        return list(source_urls)

    first_page = soup.select(first_page_query)[0]
    first_page_number = int(first_page.text)

    last_page = soup.select(last_page_query)
    if last_page:
        last_page = last_page[0]
        last_page = last_page.attrs["href"].split("/", maxsplit=4)[-1].split("/")[0]
        last_page_number = int(last_page)
    else:
        pages = []

        for a in soup.select(pages_count_query):
            text: str = a.get_text()
            if text.isdigit():
                pages.append(int(text))
        last_page_number = max(pages)

    for index in range(first_page_number, last_page_number + 1):
        if index == 1:
            continue

        target_url = urljoin(url, f"{index}/")
        source_urls.add(target_url)

    return list(source_urls)


async def get_sources_for_hotgirl_china(
    sources: list[str],
    entity: str,
    telegraph_client: Telegraph | None = None,
    final_dest: str | pathlib.Path = "",
    save_to_telegraph: bool | None = False,
    is_tag: bool | None = False,
    limit: int | None = None,
    **kwargs,
) -> None:
    tqdm_sources_iterable = tqdm(
        sources,
        total=len(sources),
        desc="Retrieving URLs...",
    )
    query, src_attr = query_mapping[entity]
    headers = headers_mapping.get(entity, None)
    page_title = ""
    title_folder_mapping = {}
    posts_sent_counter = 0
    titles = IndexedSet()
    repository = ExtractedPageRepository(model=ExtractedPage)
    ordered_unique_img_urls = None
    all_sources = []

    if final_dest:
        final_dest = pathlib.Path(final_dest)
        if not final_dest.exists():
            final_dest.mkdir(parents=True, exist_ok=True)

    for source_url in tqdm_sources_iterable:
        all_sources += await get_all_posts_for_url(repository=repository, url=source_url)

        if source_url in all_sources:
            tqdm_sources_iterable.set_description(
                f"Skipping {source_url} since it was already posted"
            )
            continue

        folder_name = ""
        urls = [*await get_images_from_pagination(url=source_url, headers=headers)]
        image_tags: list[Tag] = []

        for index, url in enumerate(urls):
            tags, soup = await get_tags(
                url,
                headers=headers,
                query=query,
            )
            image_tags.extend(tags or [])

            if index == 0:
                folder_name = (
                    soup.find("title")
                    .get_text(strip=True)
                    .split("| Hot Girl China")[0]
                    .split("/mitaku.net/")[0]
                    .strip()
                    .rstrip()
                )
                page_title = folder_name
                titles.add(page_title)

            if not page_title:
                page_title = (
                    soup.find("title")
                    .get_text(strip=True)
                    .split("| Hot Girl China")[0]
                    .split("/mitaku.net/")[0]
                    .strip()
                    .rstrip()
                )
                titles.add(page_title)

        ordered_unique_img_urls = await build_unique_img_urls(image_tags, src_attr)
        tqdm_sources_iterable.set_description(f"Finished retrieving images for {page_title}")

        if final_dest:
            final_dest = pathlib.Path(final_dest)
            if not final_dest.exists():
                final_dest.mkdir(parents=True, exist_ok=True)

            folder_name = f"{page_title}"
            title_dest = final_dest / folder_name
            # if not title_dest.exists():
                # title_dest.mkdir(parents=True, exist_ok=True)
            title_folder_mapping[page_title] = (ordered_unique_img_urls, title_dest)

        if save_to_telegraph:
            all_sources = await send_post_to_telegram(
                ordered_unique_img_urls=ordered_unique_img_urls,
                page_title=page_title,
                telegraph_client=telegraph_client,
                posts_sent_counter=posts_sent_counter,
                tqdm_sources_iterable=tqdm_sources_iterable,
                all_sources=all_sources,
                source_url=source_url,
                entity=entity,
            )
        page_title = ""

    if final_dest and ordered_unique_img_urls:
        await run_downloader(
            final_dest=final_dest,
            page_title=page_title,
            unique_img_urls=ordered_unique_img_urls,
            titles=titles,
            title_folder_mapping=title_folder_mapping,
            headers=headers,
        )
